@@ -50,6 +50,7 @@
 #include <gtkmm/togglebutton.h>
 #include <gtkmm/treeview.h>
 #include <gtkmm/menubar.h>
+#include <gtkmm/textbuffer.h>
 #include <gtkmm/adjustment.h>
 #include <gtkmm2ext/gtk_ui.h>
 #include <gtkmm2ext/click_box.h>
@@ -61,6 +62,7 @@
 #include "audio_clock.h"
 #include "ardour_dialog.h"
 #include "editing.h"
+#include "ui_config.h"
 
 class AudioClock;
 class PublicEditor;
@@ -71,9 +73,9 @@ class ConnectionEditor;
 class RouteParams_UI;
 class About;
 class AddRouteDialog;
-class NewSessionDialog;
 class LocationUI;
-class ColorManager;
+class ThemeManager;
+class NewSessionDialog;
 
 namespace Gtkmm2ext {
 	class TearOff;
@@ -93,10 +95,12 @@ namespace ALSA {
 
 #define FRAME_NAME "BaseFrame"
 
+extern sigc::signal<void>  ColorsChanged;
+
 class ARDOUR_UI : public Gtkmm2ext::UI
 {
   public:
-	ARDOUR_UI (int *argcp, char **argvp[], string rcfile);
+	ARDOUR_UI (int *argcp, char **argvp[]);
 	~ARDOUR_UI();
 
 	void show ();
@@ -127,9 +131,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 		_will_create_new_session_automatically = yn;
 	}
 
-        bool new_session(std::string path = string());
+	bool get_session_parameters (Glib::ustring path, bool have_engine = false, bool should_be_new = false);
 	gint cmdline_new_session (string path);
-	int  unload_session ();
+	int  unload_session (bool hide_stuff = false);
 	void close_session(); 
 
 	int  save_state_canfail (string state_name = "");
@@ -140,12 +144,13 @@ class ARDOUR_UI : public Gtkmm2ext::UI
         static ARDOUR::gain_t slider_position_to_gain (double pos);
 
 	static ARDOUR_UI *instance () { return theArdourUI; }
+	static UIConfiguration *config () { return ui_config; }
 
 	PublicEditor&	  the_editor(){return *editor;}
 	Mixer_UI* the_mixer() { return mixer; }
 	
 	void toggle_location_window ();
-	void toggle_color_manager ();
+	void toggle_theme_manager ();
 	void toggle_big_clock_window ();
 	void toggle_connection_editor ();
 	void toggle_route_params_window ();
@@ -156,7 +161,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	static sigc::signal<void,bool> Blink;
 	static sigc::signal<void>      RapidScreenUpdate;
 	static sigc::signal<void>      SuperRapidScreenUpdate;
-	static sigc::signal<void,nframes_t> Clock;
+	static sigc::signal<void,nframes_t, bool, nframes_t> Clock;
 
 	/* this is a helper function to centralize the (complex) logic for
 	   blinking rec-enable buttons.
@@ -198,8 +203,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void session_add_midi_track ();
 
-	void set_engine (ARDOUR::AudioEngine&);
-	gint start_engine ();
+	void create_engine ();
+	void post_engine ();
+	gint start_backend_audio ();
 
 	gint exit_on_main_window_close (GdkEventAny *);
 
@@ -213,6 +219,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void save_keybindings ();
 
 	void setup_profile ();
+	void setup_theme ();
+
+	void set_shuttle_fract (double);
 
   protected:
 	friend class PublicEditor;
@@ -286,6 +295,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	static ARDOUR_UI *theArdourUI;
 
+	void backend_audio_error ();
 	void startup ();
 	void shutdown ();
 
@@ -317,6 +327,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void about_signal_response(int response);
 
+
   private:
 	Gtk::VBox     top_packer;
 
@@ -431,7 +442,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	gint shuttle_box_expose (GdkEventExpose*);
 	gint mouse_shuttle (double x, bool force);
 	void use_shuttle_fract (bool force);
-	void set_shuttle_fract (double);
 
 	bool   shuttle_grabbed;
 	double shuttle_fract;
@@ -500,6 +510,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	Gtk::EventBox menu_bar_base;
 	Gtk::HBox     menu_hbox;
 
+	void use_menubar_as_top_menubar ();
+
 	void build_menu_bar ();
 	void build_control_surface_menu ();
 
@@ -571,6 +583,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void we_have_dependents ();
 	
 	std::string keybindings_path;
+	std::string user_keybindings_path;
 
 	void setup_keybindings ();
 	void setup_session_options ();
@@ -595,7 +608,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	int         create_location_ui ();
 	void        handle_locations_change (ARDOUR::Location*);
 
-	ColorManager* color_manager;
+	static UIConfiguration *ui_config;
+	ThemeManager *theme_manager;
 
 	/* Options window */
 	
@@ -666,12 +680,16 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void toggle_use_osc ();
 
+	void toggle_denormal_protection ();
+
 	void set_input_auto_connect (ARDOUR::AutoConnectOption);
 	void set_output_auto_connect (ARDOUR::AutoConnectOption);
 	void set_solo_model (ARDOUR::SoloModel);
 	void set_monitor_model (ARDOUR::MonitorModel);
 	void set_remote_model (ARDOUR::RemoteModel);
+	void set_denormal_model (ARDOUR::DenormalModel);
 
+	void toggle_sync_order_keys ();
 	void toggle_StopPluginsWithTransport();
 	void toggle_DoNotRunPluginsWhileRecording();
 	void toggle_VerifyRemoveLastCapture();
@@ -683,15 +701,21 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_ShowSoloMutes();
 	void toggle_LatchedRecordEnable ();
 	void toggle_RegionEquivalentsOverlap ();
+	void toggle_PrimaryClockDeltaEditCursor ();
+	void toggle_SecondaryClockDeltaEditCursor ();
+	void toggle_only_copy_imported_files ();
 
 	void mtc_port_changed ();
 	void map_solo_model ();
 	void map_monitor_model ();
+	void map_denormal_model ();
+	void map_denormal_protection ();
 	void map_remote_model ();
 	void map_file_header_format ();
 	void map_file_data_format ();
 	void map_input_auto_connect ();
 	void map_output_auto_connect ();
+	void map_only_copy_imported_files ();
 	void parameter_changed (const char*);
 
 	void set_meter_hold (ARDOUR::MeterHold);
@@ -707,6 +731,14 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void no_memory_warning ();
 	void check_memory_locking ();
+
+	void audioengine_setup ();
+
+	void display_message (const char *prefix, gint prefix_len, 
+			      Glib::RefPtr<Gtk::TextBuffer::Tag> ptag, Glib::RefPtr<Gtk::TextBuffer::Tag> mtag, const char *msg);
+	Gtk::Label status_bar_label;
+	Gtk::ToggleButton error_log_button;
+	Gtk::MessageDialog* loading_dialog;
 };
 
 #endif /* __ardour_gui_h__ */
