@@ -69,27 +69,6 @@ using boost::shared_ptr;
 
 MackieMidiBuilder builder;
 
-// Copied from tranzport_control_protocol.cc
-static inline double 
-gain_to_slider_position (ARDOUR::gain_t g)
-{
-	if (g == 0) return 0;
-	return pow((6.0*log(g)/log(2.0)+192.0)/198.0, 8.0);
-}
-
-/*
-	Copied from tranzport_control_protocol.cc
-	TODO this seems to return slightly wrong values, namely
-	with the UI slider at max, we get a 0.99something value.
-*/
-static inline ARDOUR::gain_t 
-slider_position_to_gain (double pos)
-{
-	/* XXX Marcus writes: this doesn't seem right to me. but i don't have a better answer ... */
-	if (pos == 0.0) return 0;
-	return pow (2.0,(sqrt(sqrt(sqrt(pos)))*198.0-192.0)/6.0);
-}
-
 MackieControlProtocol::MackieControlProtocol (Session& session)
 	: ControlProtocol  (session, X_("Mackie"))
 	, _current_initial_bank( 0 )
@@ -309,7 +288,8 @@ void MackieControlProtocol::switch_banks( int initial )
 		{
 			Strip & strip = *surface().strips[i];
 			// send zero for this strip
-			port_for_id(i).write( builder.zero_strip( strip ) );
+			MackiePort & port = port_for_id(i);
+			port.write( builder.zero_strip( port, strip ) );
 		}
 	}
 	
@@ -331,7 +311,7 @@ void MackieControlProtocol::switch_banks( int initial )
 
 void MackieControlProtocol::zero_all()
 {
-	// TODO turn off 55-char and SMPTE displays
+	// TODO turn off SMPTE displays
 	
 	if ( mcu_port().emulation() == MackiePort::bcf2000 )
 	{
@@ -342,11 +322,12 @@ void MackieControlProtocol::zero_all()
 	// zero all strips
 	for ( Surface::Strips::iterator it = surface().strips.begin(); it != surface().strips.end(); ++it )
 	{
-		port_for_id( (*it)->index() ).write( builder.zero_strip( **it ) );
+		MackiePort & port = port_for_id( (*it)->index() );
+		port.write( builder.zero_strip( port, **it ) );
 	}
 	
 	// and the master strip
-	mcu_port().write( builder.zero_strip( master_strip() ) );
+	mcu_port().write( builder.zero_strip( mcu_port(), master_strip() ) );
 	
 	// and the led ring for the master strip, in bcf mode
 	if ( mcu_port().emulation() == MackiePort::bcf2000 )
@@ -682,6 +663,12 @@ void MackieControlProtocol::initialize_surface()
 
 void MackieControlProtocol::close()
 {
+	// stop polling, and wait for it...
+	// must be before other shutdown otherwise polling loop
+	// calls methods on objects that are deleted
+	_polling = false;
+	pthread_join( thread, 0 );
+	
 	// TODO disconnect port active/inactive signals
 	// Or at least put a lock here
 	
@@ -739,10 +726,6 @@ void MackieControlProtocol::close()
 		_surface = 0;
 	}
 	
-	// stop polling, and wait for it...
-	_polling = false;
-	pthread_join( thread, 0 );
-	
 	// shut down MackiePorts
 	for( MackiePorts::iterator it = _ports.begin(); it != _ports.end(); ++it )
 	{
@@ -839,7 +822,7 @@ void MackieControlProtocol::handle_control_event( SurfacePort & port, Control &
 			// at which point the fader should just reset itself
 			if ( route != 0 )
 			{
-				route->set_gain( slider_position_to_gain( state.pos ), this );
+				route->gain_control().set_value( state.pos );
 				
 				// must echo bytes back to slider now, because
 				// the notifier only works if the fader is not being
@@ -971,14 +954,20 @@ void MackieControlProtocol::notify_record_enable_changed( RouteSignal * route_si
 	}
 }
 
-void MackieControlProtocol::notify_gain_changed( RouteSignal * route_signal )
+void MackieControlProtocol::notify_gain_changed( RouteSignal * route_signal, bool force_update )
 {
 	try
 	{
 		Fader & fader = route_signal->strip().gain();
 		if ( !fader.in_use() )
 		{
-			route_signal->port().write( builder.build_fader( fader, gain_to_slider_position( route_signal->route().effective_gain() ) ) );
+			float gain_value = route_signal->route().gain_control().get_value();
+			// check that something has actually changed
+			if ( force_update || gain_value != route_signal->last_gain_written() )
+			{
+				route_signal->port().write( builder.build_fader( fader, gain_value ) );
+				route_signal->last_gain_written( gain_value );
+			}
 		}
 	}
 	catch( exception & e )
@@ -1006,12 +995,9 @@ void MackieControlProtocol::notify_name_changed( void *, RouteSignal * route_sig
 				line1 = PBD::short_version( fullname, 6 );
 			}
 			
-			route_signal->port().write_sysex(
-				builder.strip_display( strip, 0, line1 )
-			);
-			route_signal->port().write_sysex(
-				builder.strip_display_blank( strip, 1 )
-			);
+			MackiePort & port = route_signal->port();
+			port.write( builder.strip_display( port, strip, 0, line1 ) );
+			port.write( builder.strip_display_blank( port, strip, 1 ) );
 		}
 	}
 	catch( exception & e )
@@ -1020,7 +1006,7 @@ void MackieControlProtocol::notify_name_changed( void *, RouteSignal * route_sig
 	}
 }
 
-void MackieControlProtocol::notify_panner_changed( RouteSignal * route_signal )
+void MackieControlProtocol::notify_panner_changed( RouteSignal * route_signal, bool force_update )
 {
 	try
 	{
@@ -1030,7 +1016,17 @@ void MackieControlProtocol::notify_panner_changed( RouteSignal * route_signal )
 		{
 			float pos;
 			route_signal->route().panner()[0]->get_effective_position( pos );
-			route_signal->port().write( builder.build_led_ring( pot, ControlState( on, pos ), MackieMidiBuilder::midi_pot_mode_dot ) );
+			
+			// cache the MidiByteArray here, because the mackie led control is much lower
+			// resolution than the panner control. So we save lots of byte
+			// sends in spite of more work on the comparison
+			MidiByteArray bytes = builder.build_led_ring( pot, ControlState( on, pos ), MackieMidiBuilder::midi_pot_mode_dot );
+			// check that something has actually changed
+			if ( force_update || bytes != route_signal->last_pan_written() )
+			{
+				route_signal->port().write( bytes );
+				route_signal->last_pan_written( bytes );
+			}
 		}
 		else
 		{
@@ -1049,19 +1045,20 @@ void MackieControlProtocol::update_automation( RouteSignal & rs )
 	ARDOUR::AutoState gain_state = rs.route().gain_automation_state();
 	if ( gain_state == Touch || gain_state == Play )
 	{
-		notify_gain_changed( &rs );
+		notify_gain_changed( &rs, false );
 	}
 	
 	ARDOUR::AutoState panner_state = rs.route().panner().automation_state();
 	if ( panner_state == Touch || panner_state == Play )
 	{
-		notify_panner_changed( &rs );
+		notify_panner_changed( &rs, false );
 	}
+	_automation_last.start();
 }
 
 void MackieControlProtocol::poll_automation()
 {
-	if ( _active )
+	if ( _active && _automation_last.elapsed() >= 20 )
 	{
 		// do all currently mapped routes
 		for( RouteSignals::iterator it = route_signals.begin(); it != route_signals.end(); ++it )
@@ -1071,6 +1068,8 @@ void MackieControlProtocol::poll_automation()
 		
 		// and the master strip
 		if ( master_route_signal != 0 ) update_automation( *master_route_signal );
+			
+		_automation_last.start();
 	}
 }
 
@@ -1088,7 +1087,7 @@ LedState MackieControlProtocol::frm_left_press( Button & button )
 	);
 	
 	// allow a quick double to go past a previous mark 
-	if ( elapsed < 500 && loc != 0)
+	if ( session->transport_rolling() && elapsed < 500 && loc != 0 )
 	{
 		Location * loc_two_back = session->locations()->first_location_before ( loc->start() );
 		if ( loc_two_back != 0 )
